#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <errno.h>
#define Malloc(type,n) (type *)malloc((n)*sizeof(type))
#define INF HUGE_VAL

struct feature_node
{
	int index;
	double value;
};


void exit_input_error(int line_num)
{
	fprintf(stderr,"Wrong input format at line %d\n", line_num);
	exit(1);
}

static char *line;
static int max_line_len;

static char* readline(FILE *input)
{
	int len;

	if(fgets(line,max_line_len,input) == NULL)
		return NULL;

	while(strrchr(line,'\n') == NULL)
	{
		max_line_len *= 2;
		line = (char *) realloc(line,max_line_len);
		len = (int) strlen(line);
		if(fgets(line+len,max_line_len-len,input) == NULL)
			break;
	}
	return line;
}

void read_problem(FILE*,FILE*);

struct feature_node *x_space;

int main(int argc, char **argv)
{
	FILE *input;

	if(argc == 2 || argc == 3){
		input = fopen(argv[1], "r");
		if(input == NULL){
			fprintf(stderr, "Can not open file %s.\n", argv[1]);
			return 1;
		}

		if(argc == 2)
			read_problem(input, stdout);
		else if(argc == 3){
			FILE* output = fopen(argv[2], "w");
			if(output == NULL){
				fprintf(stderr, "Can not open file %s.\n", argv[2]);
				fclose(input);
				return 1;
			}
			read_problem(input, output);

			fclose(output);
		}

		fclose(input);
	}else{
		fprintf(stderr, "USAGE: %s input_file [output_file = stdout]\n", argv[0]);
		return 1;
	}

	return 0;
}

void read_problem(FILE* fp, FILE* out)
{
	int i;
	long elements, j;
	char *endptr;
	char *idx, *val, *label;

	int l = 0;
	elements = 0;
	max_line_len = 1024;
	line = Malloc(char,max_line_len);
	while(readline(fp)!=NULL)
	{
		char *p = strtok(line," \t"); // label

		int elements_t = 0;
		// features
		while(1)
		{
			p = strtok(NULL," \t");
			if(p == NULL || *p == '\n')
				break;
			elements_t++;
		}
		elements_t++;

		if(elements_t > elements)
			elements = elements_t;
		l++;
	}
	rewind(fp);

	x_space = Malloc(struct feature_node,elements);

	for(i=0;i<l;i++)
	{
		j = 0;
		readline(fp);
		label = strtok(line," \t");
	
		
		//fprintf(stderr, "%f \n", prob.y[i] );

		if(endptr == label)
			exit_input_error(i+1);
		
		fprintf(out, label);

		double norm = 0;
		while(1)
		{
			idx = strtok(NULL,":");
			val = strtok(NULL," \t");

			if(val == NULL)
				break;

			errno = 0;
			x_space[j].index = (int) strtol(idx,&endptr,10);
			if(endptr == idx || errno != 0 || *endptr != '\0' || x_space[j].index <= 0)
				exit_input_error(i+1);

			errno = 0;
			x_space[j].value = strtod(val,&endptr);
			if(endptr == val || errno != 0 || (*endptr != '\0' && !isspace(*endptr)))
				exit_input_error(i+1);
			norm += x_space[j].value * x_space[j].value;

			++j;
		}

		x_space[j++].index = -1;

		norm = sqrt(norm);

		for(feature_node *x = x_space; x->index != -1; ++x){
			fprintf(out, " %d:%.16g", x->index, x->value/norm);
		}
		fprintf(out, "\n");
	}

}
