#!/usr/bin/env python

import sys

solver_labels = {5:'DCD-sh', 6:'DCD-sh', 7:'TRON',  10:'DCD-sh', 11:'DCD-sh', '10-noshrink':'DCD', '11-noshrink':'DCD', '7-noshrink':'TRON',  '5-noshrink':'DCD', '6-noshrink':'DCD', '11-cdn-shrinking3':'DCD-n', '10-cdn-shrinking3':'DCD-n', '11-cdn-shrinking5':'DCD-nnz', '10-cdn-shrinking5':'DCD-nnz', '6-cd-barQ':'DCD-1.5-sh'}
solver_ls = {5:'-', 6:'-', 10: '--', 11:'--', 12:'--', 13:'--', 7:'--', '7-noshrink':'--', '5-noshrink':'-', '6-noshrink':'-', '10-noshrink':'-', '11-noshrink':'-', '12-noshrink':':', '13-noshrink':':', '7-initial13-1e-20-noshrink': ':', '11-cdn-shrinking2':':', '10-cdn-shrinking2':':', '11-cdn-shrinking3':'--', '10-cdn-shrinking3':'--', '11-cdn-shrinking4':':', '10-cdn-shrinking4':':', '11-cdn-shrinking5':'-', '10-cdn-shrinking5':'-', '11-cdn-shrinking6':':', '10-cdn-shrinking6':':', '6-cd-barQ':':'}
solver_color = {5:'blue', 6:'blue', 10:'gray', 11:'gray', 7:'blue', '7-noshrink':'blue', '5-noshrink':'cyan', '6-noshrink':'cyan', '10-noshrink':'brown', '11-noshrink':'brown','7-initial13-1e-20-noshrink': 'blue', '11-cdn-shrinking3':'blue', '10-cdn-shrinking3':'blue', '11-cdn-shrinking5':'black', '10-cdn-shrinking5':'black', '6-cd-barQ':'black'}
solver_mark = {5:'o', 6:'o', 10:'o', 11:'o', 7:'+', '7-noshrink':'+', '5-noshrink':'', '6-noshrink':'', '10-noshrink':'', '11-noshrink':'', '11-cdn-shrinking3':'^', '10-cdn-shrinking3':'^', '11-cdn-shrinking4':'^', '11-cdn-shrinking5':'o', '10-cdn-shrinking5':'o', '6-cd-barQ':'o'}


all_data_list = {}

class Data:
	def __init__(self, name, train, test = 0, primal = 0, c = 1, p = 0.1, lower_y = 0, upper_y = 1, gamma = None):
		self.name = name
		self.train = train
		self.test = test
		self.primal = primal
		self.c = c
		self.p = p
		self.lower_y = lower_y
		self.upper_y = upper_y
		self.g = gamma

		all_data_list[name] = self

MSD_default_2 = Data('L2-MSD_default', 'YearPredictionMSD.train.yscale', 'YearPredictionMSD.test.yscale', 1.4419744195018072e+03)
MSD_scale_default_2 = Data('L2-MSD_scale_default', 'YearPredictionMSD.train.yscale.scale', 'YearPredictionMSD.test.yscale.scale', 1890.463531277353)
MSD_mse_2 = Data('L2-MSD_mse', 'YearPredictionMSD.train.yscale', 'YearPredictionMSD.test.yscale', 1.3799347990328843e+03, 0.25, 0.0009765625)
MSD_scale_mse_2 = Data('L2-MSD_scale_mse', 'YearPredictionMSD.train.yscale.scale', 'YearPredictionMSD.test.yscale.scale', 437974.4871239016, 64, 0.0009765625)
MSD_scale_lsmse_2 = Data('L2-MSD_scale_lsmse', 'YearPredictionMSD.train.yscale.scale', 'YearPredictionMSD.test.yscale.scale', None, 2, 0)

E2006_default_2 = Data('L2-E2006_default', 'E2006.train', 'E2006.test', 1663.2445964161)
E2006_scale_default_2 = Data('L2-E2006_scale_default', 'E2006.train.scale', 'E2006.test.scale', 4914.052829361523)
E2006_y1_default_2 = Data('L2-E2006_y1_default', 'E2006.train.y1', 'E2006.test.y1', 26.6536282515782, 1, .1/7.9)
E2006_mse_2 = Data('L2-E2006_mse', 'E2006.train', 'E2006.test', 65939.81437999886, 32.0, 0.0009765625)
E2006_scale_mse_2 = Data('L2-E2006_scale_mse', 'E2006.train.scale', 'E2006.test.scale', 277565.96845942, 64, 0.0009765625)
E2006_scale_lsmse_2 = Data('L2-E2006_scale_lsmse', 'E2006.train.scale', 'E2006.test.scale', None, 64, 0)

LE2006_default_2 = Data('L2-LE2006_default', 'log1p.E2006.train', 'log1p.E2006.test', 0.6782555194384958)
LE2006_scale_default_2 = Data('L2-LE2006_scale_default', 'log1p.E2006.train.scale', 'log1p.E2006.test.scale', 1042.528950545724)
LE2006_y1_default_2 = Data('L2-LE2006_y1_default', 'log1p.E2006.train.y1', 'log1p.E2006.test.y1', 0.01087477034823436, 1, .1/7.9)
LE2006_mse_2 = Data('L2-LE2006_mse', 'log1p.E2006.train', 'log1p.E2006.test', 1.002251539746436, 0.0625, 0.015625)
LE2006_scale_mse_2 = Data('L2-LE2006_scale_mse', 'log1p.E2006.train.scale', 'log1p.E2006.test.scale', 2874.72989475233, 4, 0.0009765625)
LE2006_scale_lsmse_2 = Data('L2-LE2006_scale_lsmse', 'log1p.E2006.train.scale', 'log1p.E2006.test.scale', None, 2, 0)

mydong_default_2 = Data('L2-mydong_default', 'mydong.train', 'mydong.test', 1225.311666575001)
mydong_scale_default_2 = Data('L2-mydong_scale_default', 'mydong.train.scale', 'mydong.test.scale', 2748.806196427479)
mydong_mse_2 = Data('L2-mydong_mse', 'mydong.train', 'mydong.test', 2006.549147687, 0.125, 0.015625)
mydong_scale_mse_2 = Data('L2-mydong_scale_mse', 'mydong.train.scale', 'mydong.test.scale', 301095.133235354, 16.0, 0.00390625)
mydong_scale_lsmse_2 = Data('L2-mydong_scale_lsmse', 'mydong.train.scale', 'mydong.test.scale', None, 1.0, 0)

kdd2010b_default_2 = Data('L2-kdd2010b_default', 'kdd2010b.train', 'kdd2010b.test', 242382.7379094)
kdd2010b_scale_default_2 = Data('L2-kdd2010b_scale_default', 'kdd2010b.train.scale', 'kdd2010b.test.scale', 814494.9068885891)
kdd2010b_mse_2 = Data('L2-kdd2010b_mse', 'kdd2010b.train', 'kdd2010b.test', 117545.7756496054, 0.125, 0.015625)
kdd2010b_scale_mse_2 = Data('L2-kdd2010b_scale_mse', 'kdd2010b.train.scale', 'kdd2010b.test.scale', 705490.0230701867, .5, 0.0009765625)
kdd2010b_scale_lsmse_2 = Data('L2-kdd2010b_scale_lsmse', 'kdd2010b.train.scale', 'kdd2010b.test.scale', None, .5, 0)


MSD_default_1 = Data('L1-MSD_default', 'YearPredictionMSD.train.yscale', 'YearPredictionMSD.test.yscale', 11598.57552368294)
MSD_scale_default_1 = Data('L1-MSD_scale_default', 'YearPredictionMSD.train.yscale.scale', 'YearPredictionMSD.test.yscale.scale', 12170.36272791759)
MSD_scale_mse_1 = Data('L1-MSD_scale_mse', 'YearPredictionMSD.train.yscale.scale', 'YearPredictionMSD.test.yscale.scale', 38464.57578775744, 2.0, 0.0625) # cv rate = 0.0154215
MSD_sub_scale_mse_1 = Data('L1-MSD_sub_scale_mse', 'YearPredictionMSD.train.yscale.scale.4637', 'YearPredictionMSD.test.yscale.scale', None, 32, 0.0625)
MSD_sub_scale_rbfmse_1 = Data('L1-MSD_sub_scale_rbfmse', 'YearPredictionMSD.train.yscale.scale.4637', 'YearPredictionMSD.test.yscale.scale', None, 32, 0.0625, gamma = 0.125)

E2006_default_1 = Data('L1-E2006_default', 'E2006.train', 'E2006.test', 2848.118562260584)
E2006_scale_default_1 = Data('L1-E2006_scale_default', 'E2006.train.scale', 'E2006.test.scale', 6394.967998655532)
E2006_scale_mse_1 = Data('L1-E2006_scale_mse', 'E2006.train.scale', 'E2006.test.scale', 401997.7333389374, 64.0, 0.0009765625) # cv rate =  -0.261782
E2006_scale_rbfmse_1 = Data('L1-E2006_scale_rbfmse', 'E2006.train.scale', 'E2006.test.scale', None, 64.0, 0.015625, gamma = 1) 

LE2006_default_1 = Data('L1-LE2006_default', 'log1p.E2006.train', 'log1p.E2006.test', 0.6794516888190208)
LE2006_scale_default_1 = Data('L1-LE2006_scale_default', 'log1p.E2006.train.scale', 'log1p.E2006.test.scale', 1617.052400425357)
LE2006_scale_mse_1 = Data('L1-LE2006_scale_mse', 'log1p.E2006.train.scale', 'log1p.E2006.test.scale', 2641.945377945438, 2.0, 0.0625) # cv rate = 0.133636
LE2006_scale_rbfmse_1 = Data('L1-LE2006_scale_rbfmse', 'log1p.E2006.train.scale', 'log1p.E2006.test.scale', None, 2.0, 0.0009765625, gamma=1) # cv rate = 0.133636

mydong_default_1 = Data('L1-mydong_default', 'mydong.train', 'mydong.test', 9900.276728837965 )
mydong_scale_default_1 = Data('L1-mydong_scale_default', 'mydong.train.scale', 'mydong.test.scale', 15989.59950202011)
mydong_scale_mse_1 = Data('L1-mydong_scale_mse', 'mydong.train.scale', 'mydong.test.scale', 95483.36475893654, 0.5, 0.015625) # 0.00281752
mydong_sub_scale_mse_1 = Data('L1-mydong_sub_scale_mse', 'mydong.train.scale.11382', 'mydong.test.scale', None, 0.125, 0.015625)
mydong_sub_scale_rbfmse_1 = Data('L1-mydong_sub_scale_rbfmse', 'mydong.train.scale.11382', 'mydong.test.scale', None, 0.25, 0.00390625, gamma = 1)

kdd2010b_default_1 = Data('L1-kdd2010b_default', 'kdd2010b.train', 'kdd2010b.test', 372260)
kdd2010b_scale_default_1 = Data('L1-kdd2010b_scale_default', 'kdd2010b.train.scale', 'kdd2010b.test.scale', 1659156.800208524)
kdd2010b_scale_mse_1 = Data('L1-kdd2010b_scale_mse', 'kdd2010b.train.scale', 'kdd2010b.test.scale', 3399426.942520014, 2.0, 0.0625) # cv rate = 0.0920963
kdd2010b_sub_scale_mse_1 = Data('L1-kdd2010b_sub_scale_mse', 'kdd2010b.train.scale.19264', 'kdd2010b.test.scale', None, 0.5, 0.0625)
kdd2010b_sub_scale_rbfmse_1 = Data('L1-kdd2010b_sub_scale_rbfmse', 'kdd2010b.train.scale.19264', 'kdd2010b.test.scale', None, 1, 0.015625, gamma = 1)


