#!/usr/bin/env python

import os
import stat

log_tron = 'default'
log_cdn = 'default-cdn'
output = 'default_line.py'

out = open(output, 'w')

out.write('''#!/usr/bin/env python

# THIS SCRIPT IS GENERATED BY generate_default_line.py.
# PLEASE CALL generate_default_line.py TO GENERATE THIS
# SCRIPT. OTHERWISE, ALL MODIFICATION MAY BE OVERRIDDEN.

default_obj  = {}
default_dual = {} # always empty
default_err  = {}
default_cor  = {}
default_obj_cdn  = {}
default_dual_cdn = {}
default_err_cdn  = {}
default_cor_cdn  = {}

show_line = True
show_dcd_line = True
show_cdn_line = True


def set_show_lines(val):
	global show_line, show_dcd_line, show_cdn_line
	show_line = show_dcd_line = show_cdn_line = val
	show_dcd_line = False

''')

for f in os.listdir(log_tron):

	err = None
	cor = None
	val = None

	for line in open(log_tron + '/' + f):

		line = line.strip()
		
		try:
			if 'iter' in line:
				line = line.split()
				val = float(line[9]) - float(line[3])
			elif 'Mean squared error' in line:
				err = line.split()[-2]
			elif 'Squared correlation coefficient' in line:
				cor = line.split()[-2]
		except:
			print('Can not parse data {0}'.format(f))
			break

	if not err or not cor or not val:
		print('Can not find some entries in {0}.'.format(f))
		continue

	out.write('\n#{0}\n'.format(f))
	out.write('default_obj["{0}"] = {1:.16g}\n'.format(f, val))	
	out.write('default_err["{0}"] = {1}\n'.format(f, err))	
	out.write('default_cor["{0}"] = {1}\n'.format(f, cor))	

for f in os.listdir(log_cdn):

	err = None
	cor = None
	val = None
	dual = None

	for line in open(log_cdn + '/' + f):

		try:
			if 'Primal' in line:
				val = float(line.rsplit(None, 1)[-1])
			elif 'Dual' in line:
				dual = float(line.rsplit(None, 1)[-1])
			elif 'Mean squared error' in line:
				err = line.rsplit(None, 2)[-2]
			elif 'Squared correlation' in line:
				cor = line.rsplit(None, 2)[-2]
		except:
			print('Error encountered in {0}.'.format(f))
			break

	if not err or not cor or not val or not dual:
		print('Can not find some entries in {0}.'.format(f))
		continue

	out.write('\n#{0}\n'.format(f))
	out.write('default_obj_cdn["{0}"] = {1:.16g}\n'.format(f, val))	
	out.write('default_dual_cdn["{0}"] = {1:.16g}\n'.format(f, dual))	
	out.write('default_err_cdn["{0}"] = {1}\n'.format(f, err))	
	out.write('default_cor_cdn["{0}"] = {1}\n'.format(f, cor))	



for solver in ['', '_cdn']:
	out.write('''

def get_default{0}_obj(type, data):
	if not show{0}_line:
		return None

	if type == 'primal':
		if data.name in default_obj{0}:
			return (default_obj{0}[data.name]-data.primal)/data.primal
		else:
			return None
	else:
		if data.name in default_dual{0}:
			return (default_dual{0}[data.name]+data.primal)/data.primal
		else:
			return None
'''.format(solver))

	for value in ['err', 'cor']:
		out.write('''

def get_default{1}_{0}(key):
	if not show{1}_line:
		return None

	if key in default_{0}{1}:
		return default_{0}{1}[key]
	else:
		return None
'''.format(value, solver))

out.close()
os.chmod(output, stat.S_IRWXU | stat.S_IRGRP | stat.S_IXGRP | stat.S_IROTH | stat.S_IXOTH)
