#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <errno.h>
#include "linear.h"
#define Malloc(type,n) (type *)malloc((n)*sizeof(type))
#define INF HUGE_VAL

void exit_with_help()
{
	printf(
	"Usage: train [options] training_set_file test_file\n"
	"options:\n"
	"-s type : set type of solver (default 1)\n"
	"	0 -- L2-regularized logistic regression\n"
	"	1 -- L2-loss support vector machines (dual)\n"	
	"	2 -- L2-loss support vector machines (primal)\n"
	"	3 -- L1-loss support vector machines (dual)\n"
	"	4 -- multi-class support vector machines by Crammer and Singer\n"
	"	5 -- L1-loss epsilon support vector regression(dual)\n"
	"	6 -- L2-loss epsilon support vector regression(dual)\n"
	"	7 -- L2-loss epsilon support vector regression(primal)\n"
	"	10 -- L1-loss epsilon support vector regression (CDN)\n"
	"	11 -- L2-loss epsilon support vector regression (CDN)\n"
	"-c cost : set the parameter C (default 1)\n"
	"-p epsilon : set the epsilon in loss function of epsilon-SVR (default 0.1)\n"
#ifndef NS
	"-g g -n n : to generate the experiment result of CDN with shrinking technique,\n"
	"            we run CDN with epsilon = eps/g^i, for i = 0...(n-1) (default 1.5 1)\n"
#endif
	"-e epsilon : set tolerance of termination criterion\n"
	"	-s 0 and 2\n" 
	"		|f'(w)|_2 <= eps*min(pos,neg)/l*|f'(w0)|_2,\n" 
	"		where f is the primal function, (default 0.01)\n"
	"	-s 1, 3, and 4\n"
	"		Dual maximal violation <= eps; similar to libsvm (default 0.1)\n"
	"       others\n"
	"               Dual maximal violation <= eps; similar to libsvm (default 0.01)\n"
	"-B bias : if bias >= 0, instance x becomes [x; bias]; if < 0, no bias term added (default -1)\n"
	"-wi weight: weights adjust the parameter C of different classes (see README for details)\n"
	"-v n: n-fold cross validation mode\n"
	);
	exit(1);
}

void exit_input_error(int line_num)
{
	fprintf(stderr,"Wrong input format at line %d\n", line_num);
	exit(1);
}

static char *line;
static int max_line_len;

static char* readline(FILE *input)
{
	int len;
	
	if(fgets(line,max_line_len,input) == NULL)
		return NULL;

	while(strrchr(line,'\n') == NULL)
	{
		max_line_len *= 2;
		line = (char *) realloc(line,max_line_len);
		len = (int) strlen(line);
		if(fgets(line+len,max_line_len-len,input) == NULL)
			break;
	}
	return line;
}

void parse_command_line(int argc, char **argv, char *input_file_name, char *model_file_name);
void read_problem(const char *filename);
void read_problem_test(const char *filename);
void do_cross_validation();

struct feature_node *x_space;
struct parameter param;
struct problem prob;
struct model* model_;
int flag_cross_validation;
int nr_fold;
double bias;

int main(int argc, char **argv)
{
	char input_file_name[1024];
	char test_file_name[1024];
	char model_file_name[1024];
	const char *error_msg;

	parse_command_line(argc, argv, input_file_name, test_file_name);
	read_problem(input_file_name);
	read_problem_test(test_file_name);
	error_msg = check_parameter(&prob,&param);

	if(error_msg)
	{
		fprintf(stderr,"Error: %s\n",error_msg);
		exit(1);
	}

	if(flag_cross_validation)
	{
		do_cross_validation();
	}
	else
	{
		model_=train(&prob, &param);
		//save_model(model_file_name, model_);
		destroy_model(model_);
	}
	destroy_param(&param);
	free(prob.y);
	free(prob.x);
	free(x_space);
	free(probtest.y);
	free(probtest.x);
	free(x_spacetest);

	return 0;
}

void do_cross_validation()
{
	int i;
	int total_correct = 0;
	double *target = Malloc(double, prob.l);
	double total_error = 0 ;
	double sumv = 0, sumy = 0, sumvv = 0, sumyy = 0 ,sumvy = 0 ;

	if( param.solver_type == L1SVR || param.solver_type == L2SVR || param.solver_type == L2SVRP 
	   || param.solver_type == L1SVRCDN
	   || param.solver_type == L2SVRCDN){		
		for( i = 0 ; i < prob.l;i++){
			int j = i+rand()%(prob.l-i);
			struct feature_node *tx ;
			double ty;

			tx = prob.x[i] ;
			prob.x[i]=prob.x[j] ;
			prob.x[j]=tx ;

			ty = prob.y[i] ;
			prob.y[i] = prob.y[j] ;
			prob.y[j] = ty ;
		}

		for( i=0 ; i < nr_fold ; i++ ){
			int begin = i*prob.l/nr_fold ;
			int end = (i+1)*prob.l/nr_fold ;
			int j,k ;
			struct problem subprob ; 

			subprob.l = prob.l -( end - begin ) ;
			subprob.n = prob.n ;
			subprob.x = Malloc(struct feature_node*,subprob.l);
			subprob.y = Malloc(double,subprob.l) ;



			k=0;
			for(j=0;j<begin;j++){
				subprob.x[k] = prob.x[j];								
				subprob.y[k] = prob.y[j];
				++k;
			}
			for(j=end;j<prob.l;j++){
				subprob.x[k] = prob.x[j];
				subprob.y[k] = prob.y[j];
				++k;
			}



			struct model *submodel = train(&subprob,&param);
			double error = 0;
	
			double fsumv = 0, fsumy = 0, fsumvv = 0, fsumyy = 0 ,fsumvy = 0 ;

			for(j=begin;j<end;j++){
				double v = predict(submodel,prob.x[j]);
				double y = prob.y[j];
				error += (v-y)*(v-y);
				fsumv += v;
				fsumy += y;
				fsumvv += v*v;
				fsumyy += y*y;
				fsumvy += v*y;
			}

			sumv += fsumv;
			sumy += fsumy;
			sumvv += fsumvv;
			sumyy += fsumyy;
			sumvy += fsumvy;

			destroy_model(submodel);
			printf("Mean squared error = %g\n", error/(end-begin));
			printf("Squared correlation coefficient = %g\n",
					(((end-begin)*fsumvy-fsumv*fsumy)*((end-begin)*fsumvy-fsumv*fsumy))/
					(((end-begin)*fsumvv-fsumv*fsumv)*((end-begin)*fsumyy-fsumy*fsumy))
					);
			total_error += error;
		}
		
		printf("Cross Validation Mean squared error = %g\n",total_error/prob.l);
		printf("Cross Validation Squared correlation coefficient = %g\n",
			((prob.l*sumvy-sumv*sumy)*(prob.l*sumvy-sumv*sumy))/
			((prob.l*sumvv-sumv*sumv)*(prob.l*sumyy-sumy*sumy))
				);
	}
	else{
		cross_validation(&prob,&param,nr_fold,target);
		for(i=0;i<prob.l;i++){
			if(target[i] == prob.y[i]){
				++total_correct;
			}
		}
		printf("Cross Validation Accuracy = %g%%\n",100.0*total_correct/prob.l);
	}

	free(target);
}

void parse_command_line(int argc, char **argv, char *input_file_name, char *model_file_name)
{
	int i;

	// default values
	param.solver_type = L2LOSS_SVM_DUAL;
	param.C = 1;
	param.eps = INF; // see setting below
#ifndef NS
	param.eps_g = 1.5;
	param.eps_n = 1;
#endif
	param.nr_weight = 0;
	param.p = 0.1;
	param.weight_label = NULL;
	param.weight = NULL;
	flag_cross_validation = 0;
	bias = -1;

	// parse options
	for(i=1;i<argc;i++)
	{
		if(argv[i][0] != '-') break;
		if(++i>=argc)
			exit_with_help();
		switch(argv[i-1][1])
		{
			case 's':
				param.solver_type = atoi(argv[i]);
				break;

			case 'c':
				param.C = atof(argv[i]);
				break;

			case 'p':
				param.p = atof(argv[i]);
				break;

			case 'e':
				param.eps = atof(argv[i]);
				break;

			case 'B':
				bias = atof(argv[i]);
				break;

#ifndef NS
			case 'g':
				param.eps_g = atof(argv[i]);
				break;
			case 'n':
				param.eps_n = atoi(argv[i]);
				break;
#endif
			case 'w':
				++param.nr_weight;
				param.weight_label = (int *) realloc(param.weight_label,sizeof(int)*param.nr_weight);
				param.weight = (double *) realloc(param.weight,sizeof(double)*param.nr_weight);
				param.weight_label[param.nr_weight-1] = atoi(&argv[i-1][2]);
				param.weight[param.nr_weight-1] = atof(argv[i]);
				break;

			case 'v':
				flag_cross_validation = 1;
				nr_fold = atoi(argv[i]);
				if(nr_fold < 2)
				{
					fprintf(stderr,"n-fold cross validation: n must >= 2\n");
					exit_with_help();
				}
				break;

			default:
				fprintf(stderr,"unknown option\n");
				exit_with_help();
				break;
		}
	}

	// determine filenames
	if(i>=argc)
		exit_with_help();

	strcpy(input_file_name, argv[i]);

	if(i<argc-1)
		strcpy(model_file_name,argv[i+1]);
	else
	{
		char *p = strrchr(argv[i],'/');
		if(p==NULL)
			p = argv[i];
		else
			++p;
		sprintf(model_file_name,"%s.test",p);
	}

	if(param.eps == INF)
	{
		if(param.solver_type == L2_LR || param.solver_type == L2LOSS_SVM)
			param.eps = 0.01;
		else if(param.solver_type == L1SVR || param.solver_type == L2SVR || param.solver_type == L1SVRCDN || param.solver_type == L2SVRCDN)
			param.eps = 0.1;
		else if(param.solver_type == L2SVRP)
			param.eps = 0.001;
		else if(param.solver_type == L2LOSS_SVM_DUAL || param.solver_type == L1LOSS_SVM_DUAL || param.solver_type == MCSVM_CS)
			param.eps = 0.1;
	}
}

int train_max_index = 0;

// read in a problem (in libsvm format)
void read_problem(const char *filename)
{
	int max_index, i;
	long elements, j;
	FILE *fp = fopen(filename,"r");
	char *endptr;
	char *idx, *val, *label;

	if(fp == NULL)
	{
		fprintf(stderr,"can't open input file %s\n",filename);
		exit(1);
	}

	prob.l = 0;
	elements = 0;
	max_line_len = 1024;
	line = Malloc(char,max_line_len);
	while(readline(fp)!=NULL)
	{
		char *p = strtok(line," \t"); // label

		// features
		while(1)
		{
			p = strtok(NULL," \t");
			if(p == NULL || *p == '\n')
				break;
			elements++;
		}
		elements++;
		prob.l++;
	}
	rewind(fp);

	prob.bias=bias;

	prob.y = Malloc(double,prob.l);
	prob.x = Malloc(struct feature_node *,prob.l);
	x_space = Malloc(struct feature_node,elements+prob.l);

	max_index = 0;
	j=0;
	for(i=0;i<prob.l;i++)
	{
		readline(fp);
		prob.x[i] = &x_space[j];
		label = strtok(line," \t");
		prob.y[i] = (double) strtod(label,&endptr);
		
		
		//fprintf(stderr, "%f \n", prob.y[i] );

		if(endptr == label)
			exit_input_error(i+1);

		while(1)
		{
			idx = strtok(NULL,":");
			val = strtok(NULL," \t");

			if(val == NULL)
				break;

			errno = 0;
			x_space[j].index = (int) strtol(idx,&endptr,10);
			if(endptr == idx || errno != 0 || *endptr != '\0' || x_space[j].index <= 0)
				exit_input_error(i+1);

			errno = 0;
			x_space[j].value = strtod(val,&endptr);
			if(endptr == val || errno != 0 || (*endptr != '\0' && !isspace(*endptr)))
				exit_input_error(i+1);

			++j;
		}

		if(j >= 1 && x_space[j-1].index > max_index)
			max_index = x_space[j-1].index;

		if(prob.bias >= 0)
			x_space[j++].value = prob.bias;

		x_space[j++].index = -1;
	}

	train_max_index = max_index;
	
	if(prob.bias >= 0)
	{
		prob.n=max_index+1;
		for(i=1;i<prob.l;i++)
			(prob.x[i]-2)->index = prob.n; 
		x_space[j-2].index = prob.n;
	}
	else
		prob.n=max_index;


	fclose(fp);
}
// read in a problem (in libsvm format)
void read_problem_test(const char *filename)
{
	int max_index, inst_max_index, i;
	long int elements, j;
	FILE *fp = fopen(filename,"r");
	char *endptr;
	char *idx, *val, *label;

	if(fp == NULL)
	{
		fprintf(stderr,"can't open input file %s\n",filename);
		exit(1);
	}

	probtest.l = 0;
	elements = 0;
	max_line_len = 1024;
	line = Malloc(char,max_line_len);
	while(readline(fp)!=NULL)
	{
		char *p = strtok(line," \t"); // label

		// features
		while(1)
		{
			p = strtok(NULL," \t");
			if(p == NULL || *p == '\n') // check '\n' as ' ' may be after the last feature
				break;
			elements++;
		}
		elements++;
		probtest.l++;
	}
	rewind(fp);

	probtest.bias=bias;

	probtest.y = Malloc(double,probtest.l);
	probtest.x = Malloc(struct feature_node *,probtest.l);
	x_spacetest = Malloc(struct feature_node,elements+probtest.l);

	max_index = 0;
	j=0;
	for(i=0;i<probtest.l;i++)
	{
		inst_max_index = 0; // strtol gives 0 if wrong format
		readline(fp);
		probtest.x[i] = &x_spacetest[j];
		label = strtok(line," \t");
		probtest.y[i] = (double) strtod(label,&endptr);
		
		if(endptr == label)
			exit_input_error(i+1);

		while(1)
		{
			idx = strtok(NULL,":");
			val = strtok(NULL," \t");

			if(val == NULL)
				break;

			errno = 0;
			x_spacetest[j].index = (int) strtol(idx,&endptr,10);
			if(endptr == idx || errno != 0 || *endptr != '\0' || x_spacetest[j].index <= inst_max_index)
				exit_input_error(i+1);
			else{
				if(x_spacetest[j].index > train_max_index) break;
				inst_max_index = x_spacetest[j].index;
			}

			errno = 0;
			x_spacetest[j].value = strtod(val,&endptr);
			if(endptr == val || errno != 0 || (*endptr != '\0' && !isspace(*endptr)))
				exit_input_error(i+1);

			++j;
		}

		if(inst_max_index > max_index)
			max_index = inst_max_index;

		if(probtest.bias >= 0){
			x_spacetest[j].index = train_max_index+1;
			x_spacetest[j++].value = probtest.bias;
		}

		x_spacetest[j++].index = -1;
	}

	if(probtest.bias >= 0)
	{
		probtest.n=train_max_index+1;
		//for(i=1;i<probtest.l;i++)
	//		(probtest.x[i]-2)->index = probtest.n; 
	//	x_spacetest[j-2].index = probtest.n;
	}
	else
		probtest.n=train_max_index;

	fclose(fp);
}
